'//-----------------------------------------------------------------------------------------
'//File Name:PCIELM4.bas
'//Update date:2020/01/06 by Dan Huang
'//Support Language: Visual Basic 6.0
'//-----------------------------------------------------------------------------------------
'//-----------------------------------------------------------------------------------------
'//File Name:PCIELM4.vb
'//Update date:2020/01/14 by Mike Chou
'//Support Language: Visual Basic.NET 
'//-----------------------------------------------------------------------------------------
Option Strict On
Imports System.Runtime.InteropServices

Module PCIELM4


    Public Const MAX_BOARD_NUMBER As UInt16 = 32
    Public Const MAX_EVENT_NUMBER As UInt16 = 20
    Public Const MAX_AO_CHANNEL As UInt16 = 32

    '//Option Mode
    Public Const MULTIPROCESSOR As UInt16 = 1
    Public Const ENABLEMEMIO As UInt16 = 2
    Public Const ENABLEDMAAI As UInt16 = 16

    'ModelNumber
    Public Const PCIELM4 As UInt32 = &HD61E39

    Public Const MAX_CFGCODE_NUMBER As UInt16 = 4

    'User Analog Input Config Code

    Public Const PCIELM4_AI_BI_10V As UInt16 = 0      'Bipolar +/- 10V
    Public Const PCIELM4_AI_BI_5V As UInt16 = 1       'Bipolar +/-  5V
    Public Const PCIELM4_AI_BI_2V5 As UInt16 = 2      'Bipolar +/-  2.5V
    Public Const PCIELM4_AI_BI_1V25 As UInt16 = 3     'Bipolar +/-  1.25V
    Public Const PCIELM4_AI_BI_227MV As UInt16 = 0       'Bipolar +/-  227 mV

    'AI Sampling Rate
    Public Const PCIELM4_AIRATE_15000 As UInt16 = 15000
    Public Const PCIELM4_AIRATE_7500 As UInt16 = 7500
    Public Const PCIELM4_AIRATE_3750 As UInt16 = 3750
    Public Const PCIELM4_AIRATE_1000 As UInt16 = 1000
    Public Const PCIELM4_AIRATE_500 As UInt16 = 500
    Public Const PCIELM4_AIRATE_30 As UInt16 = 30
    Public Const PCIELM4_AIRATE_15 As UInt16 = 15


    '//User Analog Output Config Code for Voltage
    Public Const PCIELM4_AO_UNI_5V As UInt16 = 0       'Unipolar 0  ~  5 V
    Public Const PCIELM4_AO_UNI_10V As UInt16 = 1      'Unipolar 0  ~ 10 V
    Public Const PCIELM4_AO_BI_5V As UInt16 = 2        'Bipolar  +/-   5 V
    Public Const PCIELM4_AO_BI_10V As UInt16 = 3       'Bipolar +/-  10 V
    Public Const PCIELM4_AO_BI_227MV As UInt16 = 0     'Bipolar +/-  227 mV



    '// Return code
    Public Const PCIELM4_NoErr As UInt16 = 0  '//Correct
    Public Const PCIELM4_OpenDriverErr As UInt16 = 1  '//Open driver error
    Public Const PCIELM4_PnPDriverErr As UInt16 = 2  '//Plug & Play error
    Public Const PCIELM4_DriverNoOpen As UInt16 = 3  '//The driver was not open
    Public Const PCIELM4_GetDriverVersionErr As UInt16 = 4  '//Recieve driver version error
    Public Const PCIELM4_ExceedBoardNumber As UInt16 = 5  '//Board number error
    Public Const PCIELM4_FindBoardErr As UInt16 = 6  '//No board found
    Public Const PCIELM4_BoardMappingErr As UInt16 = 7  '//Board Mapping error
    Public Const PCIELM4_DIOModesErr As UInt16 = 8  '//Digital input/output mode setting error
    Public Const PCIELM4_InvalidAddress As UInt16 = 9  '//Invalid address
    Public Const PCIELM4_InvalidSize As UInt16 = 10  '//Invalid size
    Public Const PCIELM4_InvalidPortNumber As UInt16 = 11  '//Invalid port number
    Public Const PCIELM4_UnSupportedModel As UInt16 = 12  '//This board model is not supported
    Public Const PCIELM4_UnSupportedFun As UInt16 = 13  '//This function is not supported
    Public Const PCIELM4_InvalidChannelNumber As UInt16 = 14  '//Invalid channel number
    Public Const PCIELM4_InvalidValue As UInt16 = 15  '//Invalid value
    Public Const PCIELM4_InvalidMode As UInt16 = 16  '//Invalid mode
    Public Const PCIELM4_GetAIStatusTimeOut As UInt16 = 17  '//A timeout occurred while receiving the status of the analog input
    Public Const PCIELM4_TimeOutErr As UInt16 = 18  '//Timeout error
    Public Const PCIELM4_CfgCodeIndexErr As UInt16 = 19  '//A compatible configuration code table index could not be found
    Public Const PCIELM4_ADCCTLTimeoutErr As UInt16 = 20  '//ADC controller a timeout error
    Public Const PCIELM4_FindPCIIndexErr As UInt16 = 21  '//A compatible PCI table index value could not be found
    Public Const PCIELM4_InvalidSetting As UInt16 = 22  '//Invalid setting value
    Public Const PCIELM4_AllocateMemErr As UInt16 = 23  '//Error while allocating the memory space
    Public Const PCIELM4_InstallEventErr As UInt16 = 24  '//Error while installing the interrupt event
    Public Const PCIELM4_InstallIrqErr As UInt16 = 25  '//Error while installing the interrupt IRQ
    Public Const PCIELM4_RemoveIrqErr As UInt16 = 26  '//Error while removing the interrupt IRQ
    Public Const PCIELM4_ClearIntCountErr As UInt16 = 27  '//Error while the clear interrupt count
    Public Const PCIELM4_GetSysBufferErr As UInt16 = 28  '//Error while retrieving the system buffer
    Public Const PCIELM4_CreateEventErr As UInt16 = 29  '//Error while create the event
    Public Const PCIELM4_UnSupportedResolution As UInt16 = 30  '//Resolution not supported
    Public Const PCIELM4_CreateThreadErr As UInt16 = 31  '//Error while create the thread
    Public Const PCIELM4_ThreadTimeOutErr As UInt16 = 32  '//Thread timeout error
    Public Const PCIELM4_FIFOOverFlowErr As UInt16 = 33  '//FIFO overflow error
    Public Const PCIELM4_FIFOTimeOutErr As UInt16 = 34  '//FIFO timeout error
    Public Const PCIELM4_GetIntInstStatus As UInt16 = 35  '//Retrieves the status of the interrupt installation
    Public Const PCIELM4_GetBufStatus As UInt16 = 36  '//Retrieves the status of the system buffer
    Public Const PCIELM4_SetBufCountErr As UInt16 = 37  '//Error while setting the buffer count
    Public Const PCIELM4_SetBufInfoErr As UInt16 = 38  '//Error while setting the buffer data
    Public Const PCIELM4_FindCardIDErr As UInt16 = 39  '//Card ID code could not be found
    Public Const PCIELM4_EventThreadErr As UInt16 = 40  '//Event Thread error
    Public Const PCIELM4_AutoCreateEventErr As UInt16 = 41  '//Error while automatically creating an event
    Public Const PCIELM4_RegThreadErr As UInt16 = 42  '//Register Thread error
    Public Const PCIELM4_SearchEventErr As UInt16 = 43  '//Search Event error
    Public Const PCIELM4_FifoResetErr As UInt16 = 44  '//Error while resetting the FIFO
    Public Const PCIELM4_InvalidBlock As UInt16 = 45  '//Invalid EEPROM block
    Public Const PCIELM4_InvalidAddr As UInt16 = 46  '//Invalid EEPROM address
    Public Const PCIELM4_AcqireSpinLock As UInt16 = 47  '//Error while acquiring spin lock
    Public Const PCIELM4_ReleaseSpinLock As UInt16 = 48  '//Error while releasing spin lock
    Public Const PCIELM4_SetControlErr As UInt16 = 49  '//Analog input setting error
    Public Const PCIELM4_InvalidChannels As UInt16 = 50  '//Invalid channel number
    Public Const PCIELM4_SearchCardErr As UInt16 = 51  '//Invalid model number
    Public Const PCIELM4_SetMapAddressErr As UInt16 = 52  '//Error while setting the mapping address
    Public Const PCIELM4_ReleaseMapAddressErr As UInt16 = 53  '//Error while releasing the mapping address
    Public Const PCIELM4_InvalidOffset As UInt16 = 54  '//Invalid memory offset
    Public Const PCIELM4_ShareHandleErr As UInt16 = 55  '//Open the share memory fail
    Public Const PCIELM4_InvalidDataCount As UInt16 = 56  '//Invalid data count
    Public Const PCIELM4_WriteEEPErr As UInt16 = 57  '//Error while writing the EEPROM
    Public Const PCIELM4_CardIOErr As UInt16 = 58  '//CardIO error
    Public Const PCIELM4_IOErr As UInt16 = 59  '//MemoryIO error
    Public Const PCIELM4_SetScanChannelErr As UInt16 = 60
    Public Const PCIELM4_SetScanConfigErr As UInt16 = 61
    Public Const PCIELM4_GetMMIOMapStatus As UInt16 = 62

    <StructLayout(LayoutKind.Sequential)> _
    Public Structure PCIELM4_DEVICE_INFO
        Public AdwSize As UInt32
        Public wVendorID As UInt16
        Public wDeviceID As UInt16
        Public wSubVendorID As UInt16
        Public wSubDeviceID As UInt16

        'dwBar(0 To 5)  As Long
        Public dwBAR0 As UInt32
        Public dwBAR1 As UInt32
        Public dwBAR2 As UInt32
        Public dwBAR3 As UInt32
        Public dwBAR4 As UInt32
        Public dwBAR5 As UInt32

        Public BusNo As Byte
        Public DevNo As Byte
        Public IRQ As Byte
        Public Aux As Byte

        'dwBarVirtualAddress(0 To 5) As Long
        Public dwBarVirtualAddress0 As UInt32 '// PCI Bar Virtual Address Bar 0(For MMIO) 
        Public dwBarVirtualAddress1 As UInt32 '// PCI Bar Virtual Address Bar 1(For MMIO)
        Public dwBarVirtualAddress2 As UInt32 '// PCI Bar Virtual Address Bar 2(For MMIO) 
        Public dwBarVirtualAddress3 As UInt32 '// PCI Bar Virtual Address Bar 3(For MMIO) 
        Public dwBarVirtualAddress4 As UInt32 '// PCI Bar Virtual Address Bar 4(For MMIO) 
        Public dwBarVirtualAddress5 As UInt32 '// PCI Bar Virtual Address Bar 5(For MMIO) 


    End Structure

    <StructLayout(LayoutKind.Sequential)> _
    Public Structure PCIELM4_CARD_INFO
        Public dwSize As UInt32             'Structure size
        Public dwModelNo As UInt32          'Model Number

        'CardID is update when calling the function each time.
        Public CardID As Byte               'for new cards, 0xFF=N/A
        Public wSingleEnded As Byte         'for new cards,1:S.E 2:D.I.F,0xFF=N/A
        Public wReserved As UInt16          'Reserver

        Public wAIChannels As UInt16        'Number of AI channels(AD)
        Public wAOChannels As UInt16        'Number of AO channels(DA)

        Public wDIPorts As UInt16           'Number of DI ports
        Public wDOPorts As UInt16           'Number of DO ports

        Public wDIOPorts As UInt16          'Number of DIO ports
        Public wDIOPortWidth As UInt16      'The width is 8/16/32 bit.

        Public wCounterChannels As UInt16   'Number of Timers/Counters
        Public wMomorySize As UInt16        'PCI-M512==>512, Units in KB.

        'dwReserved1(0 To 5) As Long  'Reserver
        Public dwReserved10 As UInt32       'Reserver 
        Public dwReserved11 As UInt32       'Reserver
        Public dwReserved12 As UInt32       'Reserver 
        Public dwReserved13 As UInt32       'Reserver 
        Public dwReserved14 As UInt32       'Reserver 
        Public dwReserved15 As UInt32       'Reserver

    End Structure


    '// Driver functions
    <DllImport("PCIELM4.dll")> _
        Public Function PCIELM4_DriverInit(ByRef wTotalBoard As UInt16) As UInt16
    End Function
    <DllImport("PCIELM4.dll")> _
        Public Function PCIELM4_DriverClose() As UInt16
    End Function
 
    '<DllImport("UniDAQ.dll", CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.StdCall, SetLastError:=True)> _
    'Public Function Ixud_GetCardInfo(ByVal wBoardNo As UInt16, ByVal a As IXUD_DEVICE_INFO, ByVal b As IXUD_CARD_INFO, ByVal szModelName As String) As UInt16

    'End Function
    Declare Ansi Function PCIELM4_GetCardInfo _
    Lib "PCIELM4.dll" (ByVal wBoardNo As UInt16, ByRef a As PCIELM4_DEVICE_INFO, ByRef b As PCIELM4_CARD_INFO, ByVal szModelName() As Byte) As UInt16
   
    'Digital I/O function
    <DllImport("PCIELM4.dll")> _
        Public Function PCIELM4_WriteDO(ByVal wBoardNo As UInt16, ByVal dwDOVal As UInt32) As UInt16
    End Function
    <DllImport("PCIELM4.dll")> _
        Public Function PCIELM4_ReadDI(ByVal wBoardNo As UInt16, ByRef dwDIVal As UInt32) As UInt16
    End Function
    'Analog Output
    <DllImport("PCIELM4.dll")> _
        Public Function PCIELM4_ConfigAO(ByVal wBoardNo As UInt16, ByVal wChannel As UInt16, ByVal wMode As UInt16, ByVal wCfgCode As UInt16) As UInt16
    End Function
    <DllImport("PCIELM4.dll")> _
        Public Function PCIELM4_WriteAOVoltage(ByVal wBoardNo As UInt16, ByVal wChannel As UInt16, ByVal fValue As Single) As UInt16
    End Function
    <DllImport("PCIELM4.dll")> _
        Public Function PCIELM4_WriteAOVoltageH(ByVal wBoardNo As UInt16, ByVal wChannel As UInt16, ByVal hValue As UInt32) As UInt16
    End Function
    <DllImport("PCIELM4.dll")> _
        Public Function PCIELM4_StartAOVoltageALL(ByVal wBoardNo As UInt16, ByVal fSamplingRate As Single, ByVal dwDataCount As UInt32, ByVal dwCyclyNum As UInt32, ByVal fValueCH0() As Single, ByVal dwValueCH1() As Single) As UInt16
    End Function
    <DllImport("PCIELM4.dll")> _
        Public Function PCIELM4_StartAOVoltageALLH(ByVal wBoardNo As UInt16, ByVal fSamplingRate As Single, ByVal dwDataCount As UInt32, ByVal dwCyclyNum As UInt32, ByVal fValueCH0() As UInt32, ByVal dwValueCH1() As UInt32) As UInt16
    End Function
    <DllImport("PCIELM4.dll")> _
        Public Function PCIELM4_StopAOALL(ByVal wBoardNo As UInt16) As UInt16
    End Function

    'Analog Input
    <DllImport("PCIELM4.dll")> _
        Public Function PCIELM4_ConfigAIAutoZero(ByVal wBoardNo As UInt16, ByVal wChannel As UInt16, ByVal wEnableAutoZero As UInt16) As UInt16
    End Function

    <DllImport("PCIELM4.dll")> _
        Public Function PCIELM4_SaveAIAutoZeroVal(ByVal wBoardNo As UInt16, ByVal wChannel As UInt16, ByVal lSamplingRate As UInt32, ByVal dwAutoZeroValue As UInt32) As UInt16
    End Function
    <DllImport("PCIELM4.dll")> _
        Public Function PCIELM4_ConfigAI(ByVal wBoardNo As UInt16, ByVal wFIFOSizeKB As UInt16, ByVal BufferSizeKB As UInt32, ByVal wCardType As UInt16, ByVal wDelaySettlingTime As UInt16, ByVal dwMode As UInt32) As UInt16
    End Function
    <DllImport("PCIELM4.dll")> _
        Public Function PCIELM4_PollingAI(ByVal wBoardNo As UInt16, ByVal wChannel As UInt16, ByVal wConfig As UInt16, ByVal lSamplingRate As UInt32, ByVal dwDataCount As UInt32, ByVal fValue() As Double) As UInt16
    End Function
    <DllImport("PCIELM4.dll")> _
        Public Function PCIELM4_PollingAIH(ByVal wBoardNo As UInt16, ByVal wChannel As UInt16, ByVal wConfig As UInt16, ByVal lSamplingRate As UInt32, ByVal dwDataCount As UInt32, ByVal dwValue() As UInt32) As UInt16
    End Function
    <DllImport("PCIELM4.dll")> _
          Public Function PCIELM4_AIHex2Vol(ByVal wBoardNo As UInt16, ByVal wChannel As UInt16, ByVal wConfig As UInt16, ByVal dwVal As UInt32, ByRef dVal As Double) As UInt16
    End Function

End Module